'Added serial interface to VGA demo to transfer screen buffer to PC
'******************************
'*  VGA 640 x 480 x 8bpp-lut  *
'******************************

CON

  intensity     = 80    '0..128

  fclk          = 80_000_000.0
  fpix          = 25_000_000.0
  fset          = (fpix / fclk * 2.0) * float($4000_0000)

  vsync         =       0       'vsync pin (all FPGA boards now)

DAT             org
              coginit #1,#@SerialMonitorOrg 
'
'               
' Setup
'
                rdfast  #0,##$1000-$400         'load .bmp palette into lut
                rep     @.end,#$100
                rflong  y
                shl     y,#8
                wrlut   y,x
                add     x,#1
.end
                rdfast  ##640*480/64,##$1000    'set rdfast to wrap on bitmap

                setxfrq ##round(fset)           'set transfer frequency to 25MHz

                setb    dira+vsync>>5,#vsync    'make vsync pin output

                'the next 4 lines may be commented out to bypass level scaling

                setcy   ##intensity << 24       'r      set colorspace for rgb
                setci   ##intensity << 16       'g
                setcq   ##intensity << 08       'b
                setcmod #%01_0_000_0            'enable colorspace conversion

                pinsetm dacmode,#0              'enable dac modes in pins 0..3
                pinsetm dacmode,#1
                pinsetm dacmode,#2
                pinsetm dacmode,#3
'
'
' Field loop
'
field           mov     x,#33                   'top blanks
                call    #blank

                mov     x,#480                  'set visible lines
line            call    #hsync                  'do horizontal sync
                xcont   m_rf,#0                 'visible line
                djnz    x,#line                 'another line?

                mov     x,#10                   'bottom blanks
                call    #blank

                notb    outa+vsync>>5,#vsync    'sync on

                mov     x,#2                    'sync blanks
                call    #blank

                notb    outa+vsync>>5,#vsync    'sync off

                jmp     #field                  'loop
'
'
' Subroutines
'
blank           call    #hsync                  'blank lines
                xcont   m_vi,#0
                djnz    x,#blank
                ret

hsync           xcont   m_bs,#0                 'horizontal sync
                xcont   m_sn,#1
                xcont   m_bv,#0
                ret
'
'
' Initialized data
'
dacmode         long    %101<<29 + %10100<<8

m_bs            long    $CF000000+16            'before sync
m_sn            long    $CF000000+96            'sync
m_bv            long    $CF000000+48            'before visible
m_vi            long    $CF000000+640           'visible

m_rf            long    $7F000000+640           'visible rlong 8bpp lut

x               res     1
y               res     1


CON  'Baud and pin settings
        SYS_CLK = 80_000_000
        BAUD_RATE = 3_000_000'115_200

        RX_PIN = 63
        TX_PIN = 62
        
dat             org  0
SerialMonitorOrg
                pinsetm pm_tx,  #TX_PIN      'set asynchronous tx mode in smart pin 
                pinsetx bitper, #TX_PIN      'set tx bit period

                pinsetm pm_rx,  #RX_PIN      'set asynchronous rx mode in smart pin 
                pinsetx bitper, #RX_PIN      'set rx bit period

                setb    dirb, #TX_PIN
                setb    dirb, #RX_PIN 

                setedg  #%01_000000+RX_PIN     'trigger on smart pin 



Waitloop           'waitx  ##10_000_000          'remove ' for delay
                'Wait for code to send screen dump
                waitedg                 'wait for smart pin  to signal rx data received
                pingetz ym,#RX_PIN            'get rx data from smart pin
                pinack  #RX_PIN              'acknowledge smart pin      
                cmp     ym,#"5"  wz,wc'"5" wz,wc  
        if_e   jmp #SendBuffer
                cmp     ym,#"A" wz,wc
        if_e   jmp #SendPalette
                'pinsety ym,#TX_PIN  'send back received character  
                jmp #WaitLoop

                 
SendPalette
                mov xm,##$1000-$400   'location of vga buffer
                mov ym,##$400  '#bytes in buffer
                jmp #SendLoop
                
SendBuffer
                mov xm,##$1000  'location of vga buffer
                mov ym,##640*480  '#bytes in buffer
SendLoop                 
                rdlong zm,xm
                'mov     zm,#"A"
                add     xm,#1                 
                pinsety zm,#TX_PIN  'send back received character
                waitx   #300     '?
                djnz    ym,#SendLoop
                jmp     #Waitloop          'loop




pm_tx          'long    %1_11_11101_0000_0000_00_0_0000000000000        'async tx long mode, dir high
                long    %1_11_11100_0000_0000_00_0_0000000000000        'async tx byte mode, dir high
pm_rx          'long    %1_10_11111_0000_0111_00_0_0000000000000        'async rx long mode, dir low, inputs pin 0
                long    %1_10_11110_0000_0000_00_0_0000000000000        'async rx byte mode, dir low, inputs pin 0 
bitper          long    SYS_CLK/BAUD_RATE                       'number of clocks per bit period, 3..65536
xm               long    0
ym               long    0
zm               long   0
im              long    0

'
' Bitmap
'
                orgh    $1000 - $436    'justify pixels at $1000, pallete at $1000-$400
                file    "bitmap2.bmp"   '640 x 480, 8pbb-lut